@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * $Id: menu.cpp 124 2008-03-23 13:32:18Z tom $
+ * $Id: menu.cpp 143 2008-07-26 18:38:59Z tom $
  */
 
 #include "menu.h"
@@ -65,9 +65,11 @@ using namespace Sudoku;
 //--- class SudokuPlugin::Menu -------------------------------------------------
 
 /** Constructor */
-Menu::Menu(cPlugin* plugin, const SetupData& setup, Puzzle& puzzle, Pos& curr) :
-  cOsdObject(true), plugin(plugin), setup(setup), puzzle(puzzle), curr(curr)
+Menu::Menu(cPlugin* plugin, const SetupData& setup, PuzzleGame*& puzzle) :
+  cOsdObject(true), plugin(plugin), setup(setup), puzzle(puzzle)
 {
+  if (puzzle == NULL)
+    puzzle = new PuzzleGame(setup.givens_count, setup.symmetric);
   xPos = (720 - GRID_SIZE) / 2;
   yPos = (576 - GRID_SIZE) / 2;
   osd = NULL;
@@ -75,9 +77,10 @@ Menu::Menu(cPlugin* plugin, const SetupData& setup, Puzzle& puzzle, Pos& curr) :
   infoText = NULL;
   new_puzzle_request = false;
 #if VDRVERSNUM >= 10504
-  maxi_font = cFont::CreateFont(Setup.FontOsd, 3 * CELL_SIZE / 4, CELL_SIZE);
-  mini_font = cFont::CreateFont(Setup.FontOsd, 3 * CELL_SIZE / RDIM / 4,
-                                CELL_SIZE / RDIM);
+  maxi_font = cFont::CreateFont(setup.maxi_font, setup.maxi_font_height,
+                                setup.maxi_font_width);
+  mini_font = cFont::CreateFont(setup.mini_font, setup.mini_font_height,
+                                setup.mini_font_width);
 #else
   maxi_font = cFont::GetFont(fontFix);
   mini_font = NULL;
@@ -146,7 +149,7 @@ eOSState Menu::ProcessKey(eKeys key)
       state = osContinue;
       const char* sudoku = list_menu->get_selected_sudoku();
       if (sudoku)
-        puzzle = Puzzle(sudoku);
+        puzzle->load_from_dump(sudoku);
       DELETENULL(list_menu);
       Show();
     }
@@ -162,6 +165,14 @@ eOSState Menu::ProcessKey(eKeys key)
       if (key == kOk)
         Setup.Save();
       DELETENULL(setup_menu);
+#if VDRVERSNUM >= 10504
+      DELETENULL(maxi_font);
+      DELETENULL(mini_font);
+      maxi_font = cFont::CreateFont(setup.maxi_font, setup.maxi_font_height,
+                                    setup.maxi_font_width);
+      mini_font = cFont::CreateFont(setup.mini_font, setup.mini_font_height,
+                                    setup.mini_font_width);
+#endif
       Show();
     }
     return state;
@@ -170,72 +181,73 @@ eOSState Menu::ProcessKey(eKeys key)
   eOSState state = cOsdObject::ProcessKey(key);
   if (state == osUnknown)
   {
-    if (key == kBack)
-      return exit();
-    if (key == kBlue)
+    Pos curr = puzzle->get_pos();
+    CommandType command = NULL;
+    switch (key)
     {
-      osd->Flush();
-      DELETENULL(osd);
-      command_menu = new CommandMenu();
-      command_menu->Display();
-      return osContinue;
+      case kLeft:
+      case kLeft|k_Repeat:
+        puzzle->set_pos(curr.prev_col());
+        break;
+      case kRight:
+      case kRight|k_Repeat:
+        puzzle->set_pos(curr.next_col());
+        break;
+      case kUp:
+      case kUp|k_Repeat:
+        puzzle->set_pos(curr.prev_row());
+        break;
+      case kDown:
+      case kDown|k_Repeat:
+        puzzle->set_pos(curr.next_row());
+        break;
+      case k0:
+      case k1:
+      case k2:
+      case k3:
+      case k4:
+      case k5:
+      case k6:
+      case k7:
+      case k8:
+      case k9:
+        puzzle->set_with_history(key - k0);
+        break;
+      case kRed:
+        command = CommandList::command(setup.key_red);
+        break;
+      case kGreen:
+        command = CommandList::command(setup.key_green);
+        break;
+      case kYellow:
+        command = CommandList::command(setup.key_yellow);
+        break;
+      case kBlue:
+        osd->Flush();
+        DELETENULL(osd);
+        command_menu = new CommandMenu();
+        command_menu->Display();
+        return osContinue;
+      case kOk:
+        if (new_puzzle_request)
+          generate();
+        break;
+      case kBack:
+        return exit();
+      default:
+        return osContinue;
     }
+    if (command)
+    {
+      state = (this->*command)();
+      if (state == osUnknown)
+        return osContinue;
+      if (state == osEnd)
+        return osEnd;
+    }
+    new_puzzle_request = !new_puzzle_request && puzzle->solved();
     if (new_puzzle_request)
-    {
-      if (key == kOk)
-        generate();
-    }
-    else
-    {
-      switch (key)
-      {
-        case kLeft:
-        case kLeft|k_Repeat:
-          curr = curr.prev_col();
-          break;
-        case kRight:
-        case kRight|k_Repeat:
-          curr = curr.next_col();
-          break;
-        case kUp:
-        case kUp|k_Repeat:
-          curr = curr.prev_row();
-          break;
-        case kDown:
-        case kDown|k_Repeat:
-          curr = curr.next_row();
-          break;
-        case k0:
-        case k1:
-        case k2:
-        case k3:
-        case k4:
-        case k5:
-        case k6:
-        case k7:
-        case k8:
-        case k9:
-          puzzle.set(curr, key - k0);
-          break;
-        case kRed:
-          puzzle.set(curr, puzzle.next_number(curr));
-          break;
-        case kGreen:
-          puzzle.toggle_mark(curr);
-          break;
-        case kYellow:
-          if (puzzle.next_free(curr) <= Pos::last())
-            curr = puzzle.next_free(curr);
-          break;
-        default:
-          return osContinue;
-      }
-    }
-    if (puzzle.solved())
-    {
-      new_puzzle_request = true;
       infoText = tr("Congratulations!\nPress OK to start a new puzzle");
-    }
     paint();
     state = osContinue;
   }
@@ -245,7 +257,7 @@ eOSState Menu::ProcessKey(eKeys key)
 /** Generate a new puzzle. */
 eOSState Menu::generate()
 {
-  puzzle.generate(setup.givens_count, setup.symmetric);
+  puzzle->generate(setup.givens_count, setup.symmetric);
   return osContinue;
 }
 
@@ -266,15 +278,52 @@ eOSState Menu::save()
   if (osd)
     osd->Flush();
   DELETENULL(osd);
-  list_menu = new ListMenu(listfile, puzzle.get_dump());
+  list_menu = new ListMenu(listfile, puzzle->get_dump());
   list_menu->Display();
   return osUnknown;
 }
 
+/** Undo last action. */
+eOSState Menu::undo()
+{
+  puzzle->backward();
+  return osContinue;
+}
+
+/** Redo last action. */
+eOSState Menu::redo()
+{
+  puzzle->forward();
+  return osContinue;
+}
+
+/** Mark/unmark the current cell. */
+eOSState Menu::toggle_mark()
+{
+  puzzle->toggle_mark(puzzle->get_pos());
+  return osContinue;
+}
+
+/** Move the cursor to the next free cell with minimal possible numbers. */
+eOSState Menu::next_cell()
+{
+  Pos new_pos = puzzle->next_cell(puzzle->get_pos());
+  if (new_pos <= Pos::last())
+    puzzle->set_pos(new_pos);
+  return osContinue;
+}
+
+/** Set the next possible number for the current cell. */
+eOSState Menu::next_number()
+{
+  puzzle->set_with_history(puzzle->next_number(puzzle->get_pos()));
+  return osContinue;
+}
+
 /** Reset the puzzle. */
 eOSState Menu::reset()
 {
-  puzzle.reset(setup.clear_marks);
+  puzzle->reset(setup.clear_marks);
   return osContinue;
 }
 
@@ -313,13 +362,13 @@ void Menu::paint()
   for (Pos p = Pos::first(); p <= Pos::last(); p = p.next())
   {
     fg = NUMBER_FG, bg = NUMBER_BG;
-    if (puzzle.given(p))
+    if (puzzle->given(p))
       fg = GIVEN_FG, bg = GIVEN_BG;
-    else if (puzzle.marked(p))
+    else if (puzzle->marked(p))
       fg = MARKED_FG, bg = MARKED_BG;
-    else if (setup.mark_errors && puzzle.error(p))
+    else if (setup.mark_errors && puzzle->error(p))
       fg = ERROR_FG, bg = ERROR_BG;
-    else if (setup.mark_ambiguous && puzzle.ambiguous(p))
+    else if (setup.mark_ambiguous && puzzle->ambiguous(p))
       fg = AMBIG_FG, bg = AMBIG_BG;
     fg = TRANS(fg, trans);
     bg = TRANS(bg, trans);
@@ -330,9 +379,9 @@ void Menu::paint()
     osd->DrawRectangle(x1, y1, x2, y2, bg);
 
     // Paint the content of the cell.
-    if (puzzle.get(p) != 0)
+    if (puzzle->get(p) != 0)
     {
-      char txt[2] = { '0' + puzzle.get(p), 0 };
+      char txt[2] = { '0' + puzzle->get(p), 0 };
       osd->DrawText(x1, y1, txt, fg, bg, maxi_font,
                     CELL_SIZE, CELL_SIZE, taCenter);
     }
@@ -340,7 +389,7 @@ void Menu::paint()
     {
       for (unsigned int n = 1; n <= DIM; ++n)
       {
-        if (puzzle.possible_number(p, n))
+        if (puzzle->possible_number(p, n))
         {
           int x3 = x1 + (((n - 1) % RDIM) * CELL_SIZE) / RDIM;
           int y3 = y1 + (((n - 1) / RDIM) * CELL_SIZE) / RDIM;
@@ -368,6 +417,7 @@ void Menu::paint()
   }
 
   // Paint the cursor.
+  Pos curr = puzzle->get_pos();
   bg = TRANS(CURSUR_COLOR, trans);
   x1 = xPos + CELL_POS(curr.col()), y1 = yPos + CELL_POS(curr.row());
   x2 = x1   + CELL_SIZE - 1,        y2 = y1   + CELL_SIZE - 1;
@@ -385,8 +435,6 @@ void Menu::paint()
     osd->DrawBitmap(xPos + 10, yPos + 10, *info);
     infoText = NULL;
   }
-  else
-    new_puzzle_request = false;
 
   osd->Flush();
 }
